/* real time linux task for reading samples from the joystick port  */
/* channel 0: X-axis from joystick 1 */
/* channel 1: Y-axis from joystick 1 */
/* channel 2: X-axis from joystick 2 */
/* channel 3: Y-axis from joystick 2 */
/* channel 4: Button 1 from joystick 1 */
/* channel 5: Button 2 from joystick 1 */
/* channel 6: Button 1 from joystick 2 */
/* channel 7: Button 2 from joystick 2 */

#define PORT GP  /* Joystick */

#include <linux/errno.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <asm/io.h> 
#include <rtl.h>
#include <rtl_sched.h>
#include <rtl_fifo.h>
#include <time.h>
#include <pthread.h>
#include "control.h"

pthread_t mytask;
int       sr;
char      channels;

/* .................................. */
/* function to write samples to fifos */
/* .................................. */
void *fun(void *t) {
  int i, data;
  unsigned char wert;
  char kan;
  hrtime_t now, later, tch[8];
  short tout[4];

  while(1) {
    if (channels==0) {
      pthread_wait_np();
    }
    else {
      kan=1;
      for (i=0; i<4; i++) {
	tch[i]=0;
	if (channels & kan) tout[i]=0;
	else tout[i]=1;
	kan*=2;
      }
      for (i=4; i<8; i++) tch[i]=100;
      now=gethrtime();
      outb(0, PORT+1); /* start timing of joystick */
      later=gethrtime();
      while ((later-now)<1900000) {
	wert=inb(PORT+1);
	later=gethrtime();
	if (wert & 1) tch[0]=later;
	else tout[0]=1;
	if (wert & 2) tch[1]=later;
	else tout[1]=1;
	if (wert & 4) tch[2]=later;
	else tout[2]=1;
	if (wert & 8) tch[3]=later;
	else tout[3]=1;
	if (!(wert & 16))  tch[4]=2148;
	if (!(wert & 32))  tch[5]=2148;
	if (!(wert & 64))  tch[6]=2148;
	if (!(wert & 128)) tch[7]=2148;
	if (tout[0] && tout[1] && tout[2] && tout[3])
	  later+=2000000;
      }
      kan=1;
      for (i=0; i<4; i++) {
	if ((channels & kan)==kan) {
	  data=(int)(tch[i]-now)/500;
	  if (rtf_put(i, (char *) &data, 2) < 0)
	    rtf_put(9, (char *) &i, sizeof(i));
	}
	kan*=2;
      }
      for (i=4; i<8; i++) {
	if ((channels & kan)==kan) {
	  if (rtf_put(i, (char *) &tch[i], 2) < 0)
	    rtf_put(9, (char *) &i, sizeof(i));
	}
	kan*=2;
      }
      pthread_wait_np();
    }
  }
}

int my_handler(unsigned int fifo) {
  struct my_msg_struct msg;
  int err, nch;

  /* FIFO 8 transfers commands to control the driver */
  while ((err=rtf_get(8, &msg, sizeof(msg))) == sizeof(msg)) {
    switch(msg.command) {
    case START_TASK:
      nch=0;
      if (channels & 1) nch++;
      if (channels & 2) nch++;
      if (channels & 4) nch++;
      if (channels & 8) nch++;
      if (channels & 16) nch++;
      if (channels & 32) nch++;
      if (channels & 64) nch++;
      if (channels & 128) nch++;
      if (nch!=0) sr=sr/nch;
      if (sr<=500) {
	pthread_make_periodic_np(mytask, gethrtime(), HRTICKS_PER_SEC*(float)(1.0/(float)sr));
	pthread_wakeup_np(mytask);
      }
      break;
    case STOP_TASK:
      pthread_suspend_np(mytask);
      break;
    case SET_CH:
      channels=(unsigned char)msg.value;
      break;
      /*
	case SET_POL:
	if ((msg.value & 1)==1) bipol[0]=1; else bipol[0]=0;
	if ((msg.value & 2)==2) bipol[1]=1; else bipol[1]=0;
	break;
	*/
    case SET_SR:
      sr=msg.value;
      break;
    default:
      return -EINVAL;
    }
  }
  if (err!=0) {
    return -EINVAL;
  }
  return 0;
}

int init_module(void) {
  pthread_attr_t     attr;
  struct sched_param sched_param;

  /* set parameters */
  sr=1;
  channels=0x00;
  rtf_create(0,10000);
  rtf_create(1,10000);
  rtf_create(2,10000);
  rtf_create(3,10000);
  rtf_create(4,10000);
  rtf_create(5,10000);
  rtf_create(6,10000);
  rtf_create(7,10000);
  rtf_create(8,100);
  rtf_create(9,100);

  pthread_attr_init(&attr);
  sched_param.sched_priority = 4;
  pthread_attr_setschedparam(&attr, &sched_param);
  pthread_create(&mytask,  &attr, fun, (void *)1);
  pthread_setfp_np(mytask,1);

  rtf_create_handler(8, &my_handler);
  return 0;
}


void cleanup_module(void)
{
  rtf_destroy(0);
  rtf_destroy(1);
  rtf_destroy(2);
  rtf_destroy(3);
  rtf_destroy(4);
  rtf_destroy(5);
  rtf_destroy(6);
  rtf_destroy(7);
  rtf_destroy(8);
  rtf_destroy(9);
  pthread_delete_np(mytask);
}



